package com.gemserk.games.discoverthename.entities;

import java.util.Collection;

import org.newdawn.slick.Animation;
import org.newdawn.slick.Color;
import org.newdawn.slick.Sound;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.entities.Entity;
import com.gemserk.componentsengine.input.InputMappingBuilderConfigurator;
import com.gemserk.componentsengine.messages.ChildrenManagementMessageFactory;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.games.discoverthename.components.AnimationRenderComponent;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.inject.Inject;
import com.google.inject.Provider;

@SuppressWarnings( { "unchecked", "unused" })
public class BlockEntityBuilder extends EntityBuilder {

	protected static final Logger logger = LoggerFactory.getLogger(BlockEntityBuilder.class);

	@Inject
	Provider<InputMappingBuilderConfigurator> inputMappingConfiguratorProvider;

	@Inject
	MessageQueue messageQueue;

	@Inject
	ChildrenManagementMessageFactory childrenManagementMessageFactory;

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;

	@Override
	public void build() {

		tags("block");

		Resource<Animation> animationResource = resourceManager.get("blockCrackAnimation");

		animationResource.get().stop();
		animationResource.get().setLooping(false);

		property("position", parameters.get("position"));
		property("animation", animationResource);
		property("breakable", parameters.get("breakable"));

		property("bounds", parameters.get("bounds"));

		Boolean breakable = Properties.getValue(entity, "breakable");
		
		property("animationColor", slick.color(1f, 1f, 1f, 1f));
		property("imageColor", slick.color(1f, 1f, 1f, 0f));

		component(new AnimationRenderComponent("renderer")).withProperties(new ComponentProperties() {
			{
				propertyRef("animation");
				propertyRef("color", "animationColor");
				property("layer", 1);
				property("size", slick.vector(1, 1));
				property("direction", slick.vector(1, 0));
			}
		});
		
		component(new ImageRenderableComponent("blockCheckedImage")).withProperties(new ComponentProperties() {
			{
				propertyRef("position");
				propertyRef("color", "imageColor");
				property("image", resourceManager.get("block"));
				property("size", slick.vector(1f, 1f));
				property("direction", slick.vector(1f, 0f));
				property("layer", 1);
			}
		});
		
		component(new ReferencePropertyComponent("startBreakingWhenBallOver") {
			
			@EntityProperty
			Property<Color> animationColor;
			
			@EntityProperty
			Property<Color> imageColor;
			
			@EntityProperty
			Property<Resource<Sound>> sound;
			
			@EntityProperty
			Property<Boolean> soundPlayed;

			@Handles
			public void onBallOver(Message message) {

				Collection<Entity> blocks = Properties.getValue(message, "blocks");

				if (!blocks.contains(entity))
					return;
				
				Sound soundValue = sound.get().get();
				
				if (!soundValue.playing() && !soundPlayed.get()) {
					soundValue.play();
					soundPlayed.set(true);
				}
				
				animationColor.get().a = 0f;
				imageColor.get().a = 1f;
				

			}

		}).withProperties(new ComponentProperties() {
			{
				property("sound", resourceManager.get("checkpoint"));
				property("soundPlayed", false);
			}
		});

	}
}
