package com.gemserk.games.discoverthename.entities;

import java.util.Collection;

import org.newdawn.slick.Animation;
import org.newdawn.slick.Sound;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.components.ReferencePropertyComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.entities.Entity;
import com.gemserk.componentsengine.input.InputMappingBuilderConfigurator;
import com.gemserk.componentsengine.messages.ChildrenManagementMessageFactory;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.Property;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.inject.Inject;
import com.google.inject.Provider;

@SuppressWarnings( { "unchecked", "unused" })
public class BreakableBlockEntityBuilder extends EntityBuilder {

	protected static final Logger logger = LoggerFactory.getLogger(BreakableBlockEntityBuilder.class);

	@Inject
	Provider<InputMappingBuilderConfigurator> inputMappingConfiguratorProvider;

	@Inject
	MessageQueue messageQueue;

	@Inject
	ChildrenManagementMessageFactory childrenManagementMessageFactory;

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;

	@Override
	public void build() {

		tags("breakable");

		parent("entities.unbreakableBlock", parameters);

		property("breaking", false);
		property("crackSound", resourceManager.get("crack"));

		component(new ReferencePropertyComponent("startBreakingWhenBallOver") {

			@EntityProperty
			Property<Boolean> breaking;
			
			@EntityProperty
			Property<Resource<Sound>> crackSound;

			@Handles
			public void onBallOver(Message message) {

				if (breaking.get())
					return;

				Collection<Entity> blocks = Properties.getValue(message, "blocks");

				if (!blocks.contains(entity))
					return;

				breaking.set(true);
				crackSound.get().get().play();

			}

		});

		component(new ReferencePropertyComponent("breakingAnimation") {

			@EntityProperty
			Property<Boolean> breaking;

			@EntityProperty
			Property<Resource<Animation>> animation;

			@Handles
			public void update(Message message) {

				if (!breaking.get())
					return;

				Resource<Animation> resource = animation.get();
				Animation animationValue = resource.get();

				if (animationValue.getFrame() == animationValue.getFrameCount() - 1) {
					messageQueue.enqueue(childrenManagementMessageFactory.removeEntity(entity));
					return;
				}

				if (animationValue.isStopped()) {
					System.out.println("restarting animation for block " + entity.getId());
					animationValue.restart();
				}
			}

		});

	}
}
